/*
 * For AM2301 download the dht library from arduino manager
 * 
 * For BH1750 download the library from github, unzip and put it in Libraries folder in arduino path, then restart ide
 * https://github.com/Genotronex/BH1750FVI_Master
 * 
 */

//Libraries
#include <DHT.h>;
#include <BH1750FVI.h> // Sensor Library
#include <Wire.h> // I2C Library
#include <SPI.h>
#include <SD.h>
#include <Sleep_n0m1.h>

Sleep sleep;
unsigned long sleepTime; //how long you want the arduino to sleep

File myFile;

//Constants

#define DHTPIN 2            //what digital pin we are connected to
#define DHTTYPE DHT21       //DHT 21  (AM2301)
DHT dht(DHTPIN, DHTTYPE);   //Initialize DHT sensor for normal 16mhz Arduino

BH1750FVI LightSensor;

//Variables
int analogPin = A0;
float hum;  //Stores humidity value
float temp; //Stores temperature value
float vlt=0.0;

void setup()
{
  sleepTime = 5000; //set sleep time in ms, max sleep time is 49.7 days
  Serial.begin(9600);
  delay(100); //wait fot serial
  dht.begin();

  LightSensor.begin();
  /*
  Set the address for this sensor
  you can use 2 different address
  Device_Address_H "0x5C"
  Device_Address_L "0x23"
  you must connect Addr pin to A3 .
  */
  LightSensor.SetAddress(Device_Address_H);//Address 0x5C
  // To adjust the slave on other address , uncomment this line
  // lightMeter.SetAddress(Device_Address_L); //Address 0x5C
  //-----------------------------------------------
  /*
   set the Working Mode for this sensor
   Select the following Mode:
    Continuous_H_resolution_Mode
    Continuous_H_resolution_Mode2
    Continuous_L_resolution_Mode
    OneTime_H_resolution_Mode
    OneTime_H_resolution_Mode2
    OneTime_L_resolution_Mode

    The data sheet recommanded To use Continuous_H_resolution_Mode
  */
  LightSensor.SetMode(Continuous_H_resolution_Mode);
  
  Serial.print("Initializing SD card...");
  if (!SD.begin(10)) {
    Serial.println("initialization failed!");
    while (1);
  }
  Serial.println("initialization done.");
  myFile = SD.open("data.csv", FILE_WRITE);
  
  // if the file opened okay, write to it:
  if (myFile) {
  myFile.println("Humidity (%RH);Temperature (C);Light (Lux);Voltage (V)");
  // close the file: 
    myFile.close();
    Serial.println("done.");
    
  } else {
    // if the file didn't open, print an error:
    Serial.println("error data.csv");
  }
}

void loop()
{
  //Read data and store it to variables hum and temp
  hum = dht.readHumidity();
  temp= dht.readTemperature();

  uint16_t lux = LightSensor.GetLightIntensity();// Get Lux value
  vlt = (2 * analogRead(analogPin) - 1024)*(5.000/1024);// Read the input pin and return the voltage value

 // open the file. note that only one file can be open at a time,
  // so you have to close this one before opening another.
  
  myFile = SD.open("data.csv", FILE_WRITE);
  // if the file opened okay, write to it:
  if (myFile) {

    myFile.print(hum);
  
    myFile.print(";");
    myFile.print(temp);

    myFile.print(";");
    myFile.print(lux);

    myFile.print(";");
    myFile.println(vlt);
    
    // close the file:
    myFile.close();
    
    Serial.println("done.");
  } else {
    // if the file didn't open, print an error:
    Serial.println("error data.csv");
  }
  // decide time interval for data retrieval: 1 sec=1000, 1 min=60000, etc
  delay(10); //Delay 10 to wait for Serial prints to complete.
  sleep.pwrDownMode(); //set sleep mode
  sleep.sleepDelay(sleepTime); //sleep for: sleepTime

}
